/*
 * This file is part of the libopencm3 project.
 *
 * Copyright (C) 2013 Toshiaki Yoshida <yoshida@mpc.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * HID Point of Sale Usage Tables
 *
 * 5 March 2001
 * Version 1.02
 */

/* POS Usage Page */
/* #define USB_HID_BAR_CODE_SCANNER_PAGE	0x8c */
#define USB_HID_WEIGHING_DEVICES_PAGE		0x8d
#define USB_HID_MAGNETIC_STRIPE_READER_PAGE	0x8e

/* Bar Code Scanner Usage Page */
#define USB_HID_BAR_CODE_BADGE_READER		0x01
#define USB_HID_BAR_CODE_SCANNER		0x02
#define USB_HID_DUMB_BAR_CODE_SCANNER		0x03
#define USB_HID_CORDLESS_SCANNER_BASE		0x04
#define USB_HID_BAR_CODE_SCANNER_CRADLE		0x05
#define USB_HID_ATTRIBUTE_REPORT		0x10
#define USB_HID_SETTINGS_REPORT			0x11
#define USB_HID_SCANNED_DATA_REPORT		0x12
#define USB_HID_RAW_SCANNED_DATA_REPORT		0x13
#define USB_HID_TRIGGER_REPORT			0x14
#define USB_HID_STATUS_REPORT			0x15
#define USB_HID_UPC_EAN_CONTROL_REPORT		0x16
#define USB_HID_EAN_2_3_LABEL_CONTROL_REPORT	0x17
#define USB_HID_CODE_39_CONTROL_REPORT		0x18
#define USB_HID_INTERLEAVED_2_OF_5_CONT_REPORT	0x19
#define USB_HID_STANDARD_2_OF_5_CONTROL_REPORT	0x1a
#define USB_HID_MSI_PLESSEY_CONTROL_REPORT	0x1b
#define USB_HID_CODABAR_CONTROL_REPORT		0x1c
#define USB_HID_CODE_128_CONTROL_REPORT		0x1d
#define USB_HID_MISC_1D_CONTROL_REPORT		0x1e
#define USB_HID_2D_CONTROL_REPORT		0x1f
#define USB_HID_AIMING_POINTER_MODE		0x30
#define USB_HID_BAR_CODE_PRESENT_SENSOR		0x31
#define USB_HID_CLASS_1A_LASER			0x32
#define USB_HID_CLASS_2_LASER			0x33
#define USB_HID_HEATER_PRESENT			0x34
#define USB_HID_CONTACT_SCANNER			0x35
#define USB_HID_EAS_NOTIFICATION		0x36
#define USB_HID_CONSTANT_EAS			0x37
#define USB_HID_ERROR_INDICATION		0x38
#define USB_HID_FIXED_BEEPER			0x39
#define USB_HID_GOOD_DECODE_INDICATION		0x3a
#define USB_HID_HANDS_FREE_SCANNING		0x3b
#define USB_HID_INTRINSICALLY_SAFE		0x3c
#define USB_HID_KLASSE_EINS_LASER		0x3d
#define USB_HID_LONG_RANGE_SCANNER		0x3e
#define USB_HID_MIRROR_SPEED_CONTROL		0x3f
#define USB_HID_NOT_ON_FILE_INDICATION		0x40
#define USB_HID_PROGRAMMABLE_BEEPER		0x41
#define USB_HID_TRIGGERLESS			0x42
#define USB_HID_WAND				0x43
#define USB_HID_WATER_RESISTANT			0x44
#define USB_HID_MULTI_RANGE_SCANNER		0x45
#define USB_HID_PROXIMITY_SENSOR		0x46
#define USB_HID_FRAGMENT_DECODING		0x4d
#define USB_HID_SCANNER_READ_CONFIDENCE		0x4e
#define USB_HID_DATA_PREFIX			0x4f
#define USB_HID_PREFIX_AIMI			0x50
#define USB_HID_PREFIX_NONE			0x51
#define USB_HID_PREFIX_PROPRIETARY		0x52
#define USB_HID_ACTIVE_TIME			0x55
#define USB_HID_AIMING_LASER_PATTERN		0x56
#define USB_HID_BAR_CODE_PRESENT		0x57
#define USB_HID_BEEPER_STATE			0x58
#define USB_HID_LASER_ON_TIME			0x59
#define USB_HID_LASER_STATE			0x5a
#define USB_HID_LOCKOUT_TIME			0x5b
#define USB_HID_MOTOR_STATE			0x5c
#define USB_HID_MOTOR_TIMEOUT			0x5d
#define USB_HID_POWER_ON_RESET_SCANNER		0x5e
#define USB_HID_PREVENT_READ_OF_BARCODES	0x5f
#define USB_HID_INITIATE_BARCODE_READ		0x60
#define USB_HID_TRIGGER_STATE			0x61
#define USB_HID_TRIGGER_MODE			0x62
#define USB_HID_T_MODE_BLINKING_LASER_ON	0x63
#define USB_HID_T_MODE_CONTINUOUS_LASER_ON	0x64
#define USB_HID_T_MODE_LASER_ON_WHILE_PULLED	0x65
#define USB_HID_T_MODE_LASER_STAYS_ON		0x66
#define USB_HID_COMMIT_PARAMETERS_TO_NVM	0x6d
#define USB_HID_PARAMETER_SCANNING		0x6e
#define USB_HID_PARAMETERS_CHANGED		0x6f
#define USB_HID_SET_PARAMETER_DEFAULT_VALUES	0x70
#define USB_HID_SCANNER_IN_CRADLE		0x75
#define USB_HID_SCANNER_IN_RANGE		0x76
#define USB_HID_AIM_DURATION			0x7a
#define USB_HID_GOOD_READ_LAMP_DURATION		0x7b
#define USB_HID_GOOD_READ_LAMP_INTENSITY	0x7c
#define USB_HID_GOOD_READ_LED			0x7d
#define USB_HID_GOOD_READ_TONE_FREQUENCY	0x7e
#define USB_HID_GOOD_READ_TONE_LENGTH		0x7f
#define USB_HID_GOOD_READ_TONE_VOLUME		0x80
#define USB_HID_NO_READ_MESSAGE			0x82
#define USB_HID_NOT_ON_FILE_VOLUME		0x83
#define USB_HID_POWERUP_BEEP			0x84
#define USB_HID_SOUND_ERROR_BEEP		0x85
#define USB_HID_SOUND_GOOD_READ_BEEP		0x86
#define USB_HID_SOUND_NOT_ON_FILE_BEEP		0x87
#define USB_HID_GOOD_READ_WHEN_TO_WRITE		0x88
#define USB_HID_GRWTI_AFTER_DECODE		0x89
#define USB_HID_GRWTI_BEEP_LAMP_AFTER_TRANSMIT	0x8a
#define USB_HID_GRWTI_NO_BEEP_LAMP_USE_AT_ALL	0x8b
#define USB_HID_BOOKLAND_EAN			0x91
#define USB_HID_CONVERT_EAN_8_TO_13_TYPE	0x92
#define USB_HID_CONVERT_UPC_A_TO_EAN_13		0x93
#define USB_HID_CONVERT_UPC_E_TO_A		0x94
#define USB_HID_EAN_13				0x95
#define USB_HID_EAN_8				0x96
#define USB_HID_EAN_99_128_MANDATORY		0x97
#define USB_HID_EAN_99_P5_128_OPTIONAL		0x98
#define USB_HID_UPC_EAN				0x9a
#define USB_HID_UPC_EAN_COUPON_CODE		0x9b
#define USB_HID_UPC_EAN_PERIODICALS		0x9c
#define USB_HID_UPC_A				0x9d
#define USB_HID_UPC_A_WITH_128_MANDATORY	0x9e
#define USB_HID_UPC_A_WITH_128_OPTIONAL		0x9f
#define USB_HID_UPC_A_WITH_P5_OPTIONAL		0xa0
#define USB_HID_UPC_E				0xa1
#define USB_HID_UPC_E1				0xa2
#define USB_HID_PERIODICAL			0xa9
#define USB_HID_PR_AUTO_DISCRIMINATE_PLUS_2	0xaa
#define USB_HID_PR_ONLY_DECODE_WITH_PLUS_2	0xab
#define USB_HID_PR_IGNORE_PLUS_2		0xac
#define USB_HID_PR_AUTO_DISCRIMINATE_PLUS_5	0xad
#define USB_HID_PR_ONLY_DECODE_WITH_PLUS_5	0xae
#define USB_HID_PR_IGNORE_PLUS_5		0xaf
#define USB_HID_CHECK				0xb0
#define USB_HID_CHECK_DISABLE_PRICE		0xb1
#define USB_HID_CHECK_ENABLE_4_DIGIT_PRICE	0xb2
#define USB_HID_CHECK_ENABLE_5_DIGIT_PRICE	0xb3
#define USB_HID_CHK_EN_EUROPEAN_4_DIGIT_PRICE	0xb4
#define USB_HID_EAN_TWO_LABEL			0xb7
#define USB_HID_EAN_THREE_LABEL			0xb8
#define USB_HID_EAN_8_FLAG_DIGIT_1		0xb9
#define USB_HID_EAN_8_FLAG_DIGIT_2		0xba
#define USB_HID_EAN_8_FLAG_DIGIT_3		0xbb
#define USB_HID_EAN_13_FLAG_DIGIT_1		0xbc
#define USB_HID_EAN_13_FLAG_DIGIT_2		0xbd
#define USB_HID_EAN_13_FLAG_DIGIT_3		0xbe
#define USB_HID_ADD_EAN_2_3_LABEL_DEFINITION	0xbf
#define USB_HID_CLEAR_ALL_EAN_2_3_LABEL_DEF	0xc0
#define USB_HID_CODABAR				0xc3
#define USB_HID_CODE_128			0xc4
#define USB_HID_CODE_39				0xc7
#define USB_HID_CODE_93				0xc8
#define USB_HID_FULL_ASCII_CONVERSION		0xc9
#define USB_HID_INTERLEAVED_2_OF_5		0xca
#define USB_HID_ITALIAN_PHARMACY_CODE		0xcb
#define USB_HID_MSI_PLESSEY			0xcc
#define USB_HID_STANDARD_2_OF_5_IATA		0xcd
#define USB_HID_STANDARD_2_OF_5			0xce
#define USB_HID_TRANSMIT_START_STOP		0xd3
#define USB_HID_TRI_OPTIC			0xd4
#define USB_HID_UCC_EAN_128			0xd5
#define USB_HID_CHECK_DIGIT			0xd6
#define USB_HID_CHECK_DIGIT_DISABLE		0xd7
#define USB_HID_CHECK_DIGIT_EN_INT_2_OF_5_OPCC	0xd8
#define USB_HID_CHECK_DIGIT_EN_INT_2_OF_5_USS	0xd9
#define USB_HID_CHECK_DIGIT_EN_STD_2_OF_5_OPCC	0xda
#define USB_HID_CHECK_DIGIT_EN_STD_2_OF_5_USS	0xdb
#define USB_HID_CHECK_DIGIT_EN_ONE_MSI_PLESSEY	0xdc
#define USB_HID_CHECK_DIGIT_EN_TWO_MSI_PLESSEY	0xdd
#define USB_HID_CHECK_DIGIT_CODABAR_ENABLE	0xde
#define USB_HID_CHECK_DIGIT_CODE_39_ENABLE	0xdf
#define USB_HID_TRANSMIT_CHECK_DIGIT		0xf0
#define USB_HID_DISABLE_CHECK_DIGIT_TRANSMIT	0xf1
#define USB_HID_ENABLE_CHECK_DIGIT_TRANSMIT	0xf2
#define USB_HID_SYMBOLOGY_IDENTIFIER_1		0xfb
#define USB_HID_SYMBOLOGY_IDENTIFIER_2		0xfc
#define USB_HID_SYMBOLOGY_IDENTIFIER_3		0xfd
#define USB_HID_DECODED_DATA			0xfe
#define USB_HID_DECODED_DATA_CONTINUED		0xff
#define USB_HID_BAR_SPACE_DATA			0x100
#define USB_HID_SCANNER_DATA_ACCURACY		0x101
#define USB_HID_RAW_DATA_POLARITY		0x102
#define USB_HID_POLARITY_INVERTED_BAR_CODE	0x103
#define USB_HID_POLARITY_NORMAL_BAR_CODE	0x104
#define USB_HID_MINIMUM_LENGTH_TO_DECODE	0x106
#define USB_HID_MAXIMUM_LENGTH_TO_DECODE	0x107
#define USB_HID_FIRST_DISCRETE_LENGTH_TO_DECODE	0x108
#define USB_HID_2ND_DISCRETE_LENGTH_TO_DECODE	0x109
#define USB_HID_DATA_LENGTH_METHOD		0x10a
#define USB_HID_DL_METHOD_READ_ANY		0x10b
#define USB_HID_DL_METHOD_CHECK_IN_RANGE	0x10c
#define USB_HID_DL_METHOD_CHECK_FOR_DISCRETE	0x10d
#define USB_HID_AZTEC_CODE			0x110
#define USB_HID_BC412				0x111
#define USB_HID_CHANNEL_CODE			0x112
#define USB_HID_CODE_16				0x113
#define USB_HID_CODE_32				0x114
#define USB_HID_CODE_49				0x115
#define USB_HID_CODE_ONE			0x116
#define USB_HID_COLORCODE			0x117
#define USB_HID_DATA_MATRIX			0x118
#define USB_HID_MAXICODE			0x119
#define USB_HID_MICROPDF			0x11a
#define USB_HID_PDF_417				0x11b
#define USB_HID_POSI_CODE			0x11c
#define USB_HID_QR_CODE				0x11d
#define USB_HID_SUPER_CODE			0x11e
#define USB_HID_ULTRA_CODE			0x11f
#define USB_HID_USD_5				0x120
#define USB_HID_VERI_CODE			0x121

/* Scale Usage Page */
#define USB_HID_WEIGHING_DEVICE			0x01
#define USB_HID_SCALE_DEVICE			0x20
#define USB_HID_SCALE_CLASS			0x21
#define USB_HID_SCALE_CLASS_I_METRIC		0x22
#define USB_HID_SCALE_CLASS_II_METRIC		0x23
#define USB_HID_SCALE_CLASS_III_METRIC		0x24
#define USB_HID_SCALE_CLASS_IIIL_METRIC		0x25
#define USB_HID_SCALE_CLASS_IV_METRIC		0x26
#define USB_HID_SCALE_CLASS_III_ENGLISH		0x27
#define USB_HID_SCALE_CLASS_IIIL_ENGLISH	0x28
#define USB_HID_SCALE_CLASS_IV_ENGLISH		0x29
#define USB_HID_SCALE_CLASS_GENERIC		0x2a
#define USB_HID_SCALE_ATTRIBUTE_REPORT		0x30
#define USB_HID_SCALE_CONTROL_REPORT		0x31
#define USB_HID_SCALE_DATA_REPORT		0x32
#define USB_HID_SCALE_STATUS_REPORT		0x33
#define USB_HID_SCALE_WEIGHT_LIMIT_REPORT	0x34
#define USB_HID_SCALE_STATISTICS_REPORT		0x35
#define USB_HID_DATA_WEIGHT			0x40
#define USB_HID_DATA_SCALING			0x41
#define USB_HID_WEIGHT_UNIT			0x50
#define USB_HID_WEIGHT_UNIT_MILLIGRAM		0x51
#define USB_HID_WEIGHT_UNIT_GRAM		0x52
#define USB_HID_WEIGHT_UNIT_KILOGRAM		0x53
#define USB_HID_WEIGHT_UNIT_CARATS		0x54
#define USB_HID_WEIGHT_UNIT_TAELS		0x55
#define USB_HID_WEIGHT_UNIT_GRAINS		0x56
#define USB_HID_WEIGHT_UNIT_PENNYWEIGHTS	0x57
#define USB_HID_WEIGHT_UNIT_METRIC_TON		0x58
#define USB_HID_WEIGHT_UNIT_AVOIR_TON		0x59
#define USB_HID_WEIGHT_UNIT_TROY_OUNCE		0x5a
#define USB_HID_WEIGHT_UNIT_OUNCE		0x5b
#define USB_HID_WEIGHT_UNIT_POIND		0x5c
#define USB_HID_CALIBRATION_COUNT		0x60
#define USB_HID_REZERO_COUNT			0x61
#define USB_HID_SCALE_STATUS			0x70
#define USB_HID_SS_FAULT			0x71
#define USB_HID_SS_STABLE_AT_CENTER_OF_ZERO	0x72
#define USB_HID_SS_IN_MOTION			0x73
#define USB_HID_SS_WEIGHT_STABLE		0x74
#define USB_HID_SS_UNDER_ZERO			0x75
#define USB_HID_SS_OVER_WEIGHT_LIMIT		0x76
#define USB_HID_SS_REQUIRES_CALIBRATION		0x77
#define USB_HID_SS_REQUIRES_REZEROING		0x78
#define USB_HID_ZERO_SCALE			0x80
#define USB_HID_ENFORCED_ZERO_RETURN		0x81

/* Magnetic Stripe Reader (MSR) Usage Page */
#define USB_HID_MSR_DEVICE_READ_ONLY		0x01
#define USB_HID_TRACK_1_LENGTH			0x11
#define USB_HID_TRACK_2_LENGTH			0x12
#define USB_HID_TRACK_3_LENGTH			0x13
#define USB_HID_TRACK_JIS_LENGTH		0x14
#define USB_HID_TRACK_DATA			0x20
#define USB_HID_TRACK_1_DATA			0x21
#define USB_HID_TRACK_2_DATA			0x22
#define USB_HID_TRACK_3_DATA			0x23
#define USB_HID_TRACK_JIS_DATA			0x24
